#!/usr/bin/python
# -*- coding: utf-8 -*-

import cherrypy
import os
import dircache
import sys
from kclient import kiwi, localdb, fileIterator, configManager, connectionManager
from kclient.backupOs import bos
import demjson
import subprocess
import string
import logging
import datetime
import time
import utils
import codecs
import md5
import re
import feedparser
import socket
import pycurl

if bos.isWin32():
    import win32con
    from kclient.win_utils import safe_mount_remote

logging.basicConfig(level=logging.INFO,
                    format='%(asctime)s %(levelname)-8s %(module)s %(message)s',
                    datefmt='%a, %d %b %Y %H:%M:%S',
                    filename=os.path.join(bos.getCurrentDir(),'kiwi-server.log'),
                    filemode='a')


class KiwiManager:

    def __init__( self, configname, cur_dir ):
        self.kernel = kiwi.Kiwi(configname) 
        self.current_dir = cur_dir
        #cherrypy.log("Starting service in "+cur_dir, context='', severity=logging.WARNING)
        utils.unzipTo(os.path.join(cur_dir,'webclient/static.zip'), os.path.join(cur_dir, 'webclient'))
        utils.unzipTo(os.path.join(cur_dir,'webclient/custom.zip'), os.path.join(cur_dir, 'webclient'))
        dircache.reset()
        self._registered = 0
        self.checkRegisterState()
        if bos.isWin32():
            safe_mount_remote(self.kernel.config)
        
        
    @cherrypy.expose
    def getRSS(self, **kwargs):
        self.kernel.config.reload()

        rssUrl2 = 'http://blog.kiwi-backup.com/category/trucs-et-astuces/feed/'

        rssUrl = self.kernel.config.getSection('params').get('rssUrl', rssUrl2)
         
        cherrypy.response.headers['Content-Type'] = 'application/json; charset=UTF-8'
        
        try:
            feed = feedparser.parse(rssUrl)
        except feedparser.NonXMLContentType, e:
            feed = feedparser.parse(rssUrl2)
        except not feedparser.NonXMLContentType, e:
            return ''
        try:
            return demjson.encode(feed).encode('utf8')
        except Exception, e:
            return ''
            
        
    @cherrypy.expose
    def checkRegisterState(self):
        _conf = self.kernel.config
        _conf.reload()
        uid = _conf.getSection('params')['clientid']
        self._registered = uid and uid != 'X' and uid != 'x'
        return self._registered
    
    @cherrypy.expose
    def index(self):
        cherrypy.response.headers['Content-Type'] = 'text/html; charset=UTF-8'
        if bos.isOsX() :
            vrs = {'mac_BEGIN' : '',  'mac_END': ''}
        else:
            vrs = {'mac_BEGIN' : '<!--',  'mac_END': '-->'}
        return self.parseTpl('index', vrs)

    @cherrypy.expose
    def blank(self):
        return ""
    
    @cherrypy.expose
    def backup_config(self):
        cherrypy.response.headers['Content-Type'] = 'text/truc; charset=utf-8'

        self.kernel.config.reload()
        cvars = self.kernel.config.getSection('files')
        vrs = {}
        incl = cvars['include']
        include = []
        for pth in incl:
            include.append({'id':pth, 'path':pth})
 
        jsoninclude = demjson.encode(include)
        vrs['include'] = jsoninclude.encode('utf8')
        
        excl = cvars['exclude']
        exclude = []
        for pth in excl:
            exclude.append({'id':pth, 'path':pth})
 
        jsonexclude = demjson.encode(exclude)
        
        vrs['exclude'] = jsonexclude.encode('utf8')
        vrs['excludepattern'] = "\n".join(cvars['excludepattern'])
        
        vrs['hour'] = self.kernel.config.getSection('scheduler')['hour']
        vrs['minute'] = self.kernel.config.getSection('scheduler')['min']

        vrs['shutDownAfterBackup'] = self.kernel.config.getRootSection()['kiwi'].get('shutdown_after_backup', False)

        if( self.kernel.config.getSection('files').get('use_vss', 0) == 1):
            vrs['use_vss'] = True
        else:
            vrs['use_vss'] = False

        vrs['max_recv_speed'] = self.kernel.config.getSection('security').get('max_recv_speed', '')
        vrs['max_send_speed'] = self.kernel.config.getSection('security').get('max_send_speed', '')

        return self.parseTpl('backup_config', vrs )

    @cherrypy.expose
    def restore_config(self):

        cherrypy.response.headers['Content-Type'] = 'text/html; charset=UTF-8'
        if bos.isOsX():
            vrs = {'destpath': '/Users/Shared/restore'}
        else: 
            vrs = {'destpath':os.path.join(self.current_dir, 'restore')}
        return self.parseTpl('restore_config', vrs )


    @cherrypy.expose
    def accueil(self):
        cherrypy.response.headers['Content-Type'] = 'text/html; charset=UTF-8'
        vars = {}
        if self.checkRegisterState():
            if not os.path.exists(os.path.join(self.current_dir, 'tmp/dbm.db')):
                return "<h2>No database, please run synchro</h2>"
            ldb = localdb.Localdb(os.path.join(self.current_dir, 'tmp/dbm.db'))
            vars = ldb.getInfos()
            cfg = self.kernel.config.getRootSection()['kiwi']
            vars['uid'] = cfg['params']['clientid']

            vars['client_version'] = self.kernel.clientVersion

            stats = ldb.getStats()
            if stats:
                stats = stats[0]
                vars['date'] = datetime.datetime.strptime(stats[0], '%Y-%m-%d %H:%M:%S').strftime('%d/%m/%Y %Hh %Mm %Ss')
                vars['send_tar_file_size'] = utils.sizeToHuman(stats[5])
                vars['source_file_size'] =  utils.sizeToHuman(stats[4])
                vars['source_files'] = stats[3]
                if stats[4]:
                    vars['ratio'] =  '%.2f ' % ( float(stats[5]) / float(stats[4]) * 100.0 )
                else:    
                    vars['ratio'] =  '--'
                
                #vars['elapsed_time'] = '%.2f s' % stats[2]
                vars['elapsed_time'] = utils.secondsToTime(stats[2])
                
            else:
                vars['date'] = '--'
                vars['send_tar_file_size'] = '--'
                vars['source_file_size'] =  '--'
                vars['source_files'] = '--'
                vars['ratio'] = '--'
                vars['elapsed_time'] = '--'

            return self.parseTpl('accueil_registered', vars )
        else:
            vars['kserver_hostname'] = ", ".join(['%s' % (host) for host in self.kernel.config.getSection('params')['servers']])
            return self.parseTpl('accueil_notregistered', vars )
 
    @cherrypy.expose
    def register(self, user, key, server_hostname ):
        if not self.checkRegisterState():
            status = self.kernel.runInstall( str(user), str(key), str(server_hostname) )
            if status[0]:
                return demjson.encode({
                    'success': True,
                    'status': 'ok',
                    'uid': self.kernel.config.getSection('params')['clientid'],
                    'text': 'coucou'
                })
            else:
                return demjson.encode({ 'success': False,
                        'status': False,
                        'params': status[1] })
        else:
            return demjson.encode({ 'success': False,
                    'status': 'ok',
                    'params': { 'msg': 'already registered'} } )
            
    def parseTpl(self, filename, vrs = {}):
        path = self.current_dir + '/webclient/static/%s.tpl' % filename
        tpl = string.Template(file(path, 'r').read())
        html = tpl.safe_substitute(vrs)
        return html

    def _getHTML(self, filename = 'index'):
        path = self.current_dir+'/webclient/static/%s.html' % filename
        return file(path, 'r' ).read()
    
    def _formatFileList(self, flist ):
        ret = ''
        for fichier in flist:
            ret += fichier.encode('utf-8') + bos.getCR()
        
        return ret
    
    def _getStatus(self, path, ldb):
        if ldb:
            return ldb.getStatus(path)
        else:
            return 0
        
    def _getAction(self, path):
        _conf = self.kernel.config.getSection('files')

        if path in _conf['include'] and not path in _conf['exclude']:
            return True
        
        return False
    
    def _getIconClass(self, isBackuped ):
        if isBackuped == 1:
            cls = 'folder-backuped'
        elif isBackuped == 2:
            cls = 'child-backuped'
        else:
            cls = ''
            
        return cls
        
    @cherrypy.expose
    def getNodeChildren(self, node):

        cherrypy.response.headers['Content-Type'] = 'text/json; charset=UTF-8'
        dbname = os.path.join(self.current_dir, 'tmp/dbm.db')
        if os.path.exists(dbname):
            ldb = localdb.Localdb(dbname)
        else:
            ldb = None
        
        paths = []
        
        if node == '__root__':
            roots = bos.getInitialDirectory()
            for root in roots:
                bpath = root
                if root == '':
                    root = '/'
                    bpath = root
                else:
                    root = '/'+ root +'/'
                
                checked = self._getAction(root)
                isBackuped = 0
                
                paths.append({
                    'data' : {
                      'title': unicode( bpath, 'utf-8' ),
                      'icon' : 'folder',
                      'attr' : {
                        'reallyChecked': checked,
                        'class': 'draggable '+self._getIconClass(isBackuped),
                        'path' : unicode( root, 'utf-8' )
                      }
                    },
                    'attr' : {
                      'id': unicode(root, 'utf-8' ),
                      'node' : unicode( root, 'utf-8' )
                      },
                    'state' : 'closed'
                })
        
        else:
            unode = bos.toOsFilename(node)
            for f in dircache.listdir(unode):
                if isinstance(f, unicode):
                    abspath = os.path.join(unode, f)
                else:
                    abspath = os.path.join(unode, unicode(f,'utf8','replace'))
                bpath = bos.toBackupFilename(abspath)
                isLeaf = os.path.isfile(abspath) or os.path.islink(abspath)
                checked = self._getAction(bpath)
                isBackuped = self._getStatus(bpath, ldb)
                cssClass  = self._getIconClass(isBackuped)
                cssClass += ' leaf ' if(isLeaf and not isBackuped) else ' folder '
                
                #if os.path.isdir(abspath):
                paths.append({
                    'data' : {
                        'title': f,
                        'icon':  cssClass,
                        'attr': {
                            'path': abspath,
                            'leaf': isLeaf,
                            'reallyChecked': checked,
                            'class' : 'draggable '
                        }
                    },
                    'attr' : {
                        'id': bpath,
                        'node': bpath,
                    },
                    'state' : '' if (isLeaf) else 'closed'

                })
                    
        return demjson.encode(paths, strict=False, escape_unicode=True)
   
    
    @cherrypy.expose
    def getFiles(self, path):
        dbname = os.path.join(self.current_dir, 'tmp/dbm.db')
        if os.path.exists(dbname):
            ldb = localdb.Localdb(dbname)
        else:
            ldb = None
        
        paths = []
            
        node = bos.toOsFilename(unicode(path, 'utf-8'))
        for f in dircache.listdir(node):
            abspath = os.path.join(node, f)
            bpath = bos.toBackupFilename(abspath)
            isLeaf = os.path.isfile(abspath) or os.path.islink(abspath)
            checked = self._getAction(bpath)
            isBackuped = self._getStatus(bpath, ldb)
            if isLeaf:
                paths.append({
                       'id': bpath,
                       'name': f,
                       'date': 0,
                       'type': 'Fichier',
                       'checked': checked,
                       'status': isBackuped,
                       'reallyChecked': checked,
                       'iconCls': self._getIconClass(isBackuped)
                })
                    
        return demjson.encode({ 'files': paths }, encoding="utf8")

    @cherrypy.expose
    def getBFiles(self, path):
        
        p = path.split('__', 1)
        if len(p) == 2:
            path = p[1]
            bid = p[0]
        else:
            bid = 0

        ldb = localdb.Localdb(os.path.join(self.current_dir, 'tmp/dbm.db'))
        paths = []
            
        node = bos.toOsFilename(unicode(path, 'utf-8'))
        for f in ldb.getFiles(path, bid, recursive=0):
            if f['type'] != 'Dir' and f['path'] != path:
                abspath = os.path.join(node, f['path'])
                bpath = bos.toBackupFilename(abspath)
                checked = self._getAction(bpath)
                cssclass = 'folder-backuped'
                paths.append({
                       'id': bpath,
                       'name': os.path.basename(f['path']),
                       'date': 0,
                       'type': 'Fichier',
                       'checked': checked,
                       'status': 1,
                       'reallyChecked': checked,
                       'iconCls': cssclass
                })
                        
        return demjson.encode({ 'files': paths }, encoding = "utf8")
 
    def getBids(self):
        backups = []
        ldb = localdb.Localdb(os.path.join(self.current_dir, 'tmp/dbm.db'))
        #locale.setlocale(locale.LC_ALL, '')
        format = '%Y-%m-%d %H:%M:%S' #locale.nl_langinfo(locale.D_FMT)
        for b in ldb.getBackups():
            localDate = datetime.datetime.strptime(b[1], format)
            backups.append({
                        'attr': {
                          'id':   '__root_%s__' % b[0],
                          'node': '__root_%s__' % b[0]
                        },
                        'data': {
                          'attr': {
                            'leaf' : False,
                            'path': '__root__',
                          },
                          'bid': b[0],
                          'title': localDate.strftime('%d/%m/%Y %H:%M:%S'),
                          'icon': 'backup-tree'
                        },
                        'state': 'closed'
                    })
        return demjson.encode(backups, encoding = 'utf8')
    
    @cherrypy.expose
    def getBackupNodeChildren(self, node):
        cherrypy.response.headers['Content-Type'] = 'text/json; charset=UTF-8'
        paths = []
        
        if(node == '__bid_root__'):                     # On veut la liste des backups, path = '', bid = ''
            path = ''
            bid = ''
        elif(node.startswith('__root_')):               # On veut la racine d'un backup {bid}
            matches = re.search('__root_([^_]+)__$', node)
            path = ''
            bid = matches.group(1)
        else:                                           # on veut une feuille
            matches = re.search('([^_]+)__(.*)$', node)
            bid = matches.group(1)
            path = matches.group(2)
        
        if isinstance(path, unicode):
            upath = path
        else:
            upath = unicode(path,'utf8','replace')
            
        ldb = localdb.Localdb(os.path.join(self.current_dir, 'tmp/dbm.db'))
        date_backup = ''
        if ldb.getBackup(bid):
            localDate = datetime.datetime.strptime(ldb.getBackup(bid)[1], '%Y-%m-%d %H:%M:%S')
            date_backup = localDate.strftime('%d/%m/%Y %H:%M:%S')
        if node == '__bid_root__':
            return self.getBids()
        if node.startswith('__root_'):
            for f in ldb.getRootDir():
                _file = os.path.split(f['path'])[1]
                bpath = bos.toBackupFilename(f['path'])
                ospath = bos.toBackupFilename(f['path'])
                        
                isLeaf = False
                if (f['type'] != 'Dir'): isLeaf = True
                else : isLeaf = False
                
                #checked = self._getAction(ospath)
                if not upath == '__root__':
                    text = _file
                else:
                    text = bpath
 
                paths.append({
                        'attr': {
                          'node': bid+'__'+bpath
                        },
                        'data': {
                          'attr': {
                            'id':   bid+'__'+bpath,
                            'class' : 'draggable',
                            'leaf' : isLeaf,
                            'path': bpath,
                            'date_backup': date_backup,
                            'bid': bid,
                          },
                          'title': text,
                          'icon' : 'leaf' if (isLeaf) else 'folder'
                        },
                        'state': '' if (isLeaf) else 'closed'
                        #'checked': False
                    })
        else: 
            for f in ldb.getFiles(upath, bid, recursive = 0):
                if upath != f['path']:
                    _file = os.path.split(f['path'])[1]
                    bpath = bos.toBackupFilename(f['path'])
                    ospath = bos.toBackupFilename(f['path'])
                    if (f['type'] != 'Dir'): isLeaf = True
                    else : isLeaf = False
                    checked = self._getAction(ospath)
                    if not upath == '__root__':
                        text = _file
                    else:
                        text = bpath
                        
                    paths.append({
                        'attr': {
                          'node': bid+'__'+bpath,
                        },
                        'data': {
                          'attr': {
                            'id':   bid+'__'+bpath,
                            'class' : 'draggable',
                            'leaf' : isLeaf,
                            'path': bpath,
                            'date_backup': date_backup,
                            'bid': bid,
                          },
                          'title': text,
                          'icon' : 'leaf' if (isLeaf) else 'folder'
                        },
                        'state' : '' if (isLeaf) else 'closed'
                        #'checked': False
                    })
                
        return demjson.encode(paths, strict=True, encoding = 'utf8')
    
    
    @cherrypy.expose
    def getVersions( self, path ):
        
        versions = []
        ldb = localdb.Localdb(os.path.join(self.current_dir, 'tmp/dbm.db'))
        for v in ldb.getVersions(path):
            versions.append({'vid': v[0], 'date': v[1], 'type': v[2]})

        
        return demjson.encode({
            'versions': versions
        })
   
    def getExecutable(self):
        executable = ['./kiwi.py']
        if bos.isWin32():
            if os.path.exists(os.path.join(self.current_dir, 'kiwi.exe')): 
                executable = [os.path.join(self.current_dir, "kiwi.exe")]
            if os.path.exists(os.path.join(self.current_dir, 'kiwi.py')): 
                executable = ["python.exe", os.path.join(self.current_dir, "kiwi.py")]
        elif bos.isOsX():
            if os.path.exists(os.path.join(self.current_dir, 'kiwi.py')): 
                executable = [os.path.join(self.current_dir, 'kiwi.py')]
            else: executable = ['../MacOS/server']

            
        else:
            if os.path.exists('./kiwi'): 
                executable = ["./kiwi"]
            if os.path.exists(os.path.join(self.current_dir, 'kiwi.py')): 
                executable = [os.path.join(self.current_dir, "kiwi.py")]
        return executable

        
    @cherrypy.expose
    def runBackup(self, **kargs):
        cherrypy.response.headers['Content-Type'] = 'text/html'
        executable = self.getExecutable()
        cmd = executable + ["-b"]+ ["-v"]
        #cherrypy.log("Running "+str(cmd), context='', severity=logging.ERROR)
        stdout_file = codecs.open(os.path.join(self.current_dir,'stdout.tmp'), 'w', encoding='utf8', buffering=0)
        if bos.isWin32():
            startupinfo = subprocess.STARTUPINFO()
            startupinfo.dwFlags |= subprocess.STARTF_USESHOWWINDOW
            startupinfo.wShowWindow = win32con.SW_HIDE
        
            proc = subprocess.Popen(cmd,
                        bufsize=0,
                        cwd = self.current_dir,
                        stdout=stdout_file,
                        stderr=subprocess.STDOUT,
                        startupinfo=startupinfo)
        else:
            proc = subprocess.Popen(cmd,
                        bufsize=0,
                        cwd = self.current_dir,
                        stdout=stdout_file,
                        stderr=subprocess.STDOUT)
 

        def content(proc):
            
            header = """<html>
            <body style="background-color:black;color:white;">

            """             
            yield header
            retval = None
            stdout_file = open(os.path.join(self.current_dir,'stdout.tmp'), 'rb', 0)
            stdout_file.seek(0)
            while retval == None:
                retval = proc.poll()
                where = stdout_file.tell()
                s = stdout_file.read()
                if s:
                    yield s.replace("\n", "<br />\n")+'<script>window.scroll(0,500000);</script>'
                else:
                    time.sleep(1)
                    stdout_file.seek(where)

            s = stdout_file.read()
            yield s.replace("\n", "<br />\n")
            if (retval == 0): mess = 'Ok'
            else: mess = 'Error '+str(retval)
            yield '<hr />' + mess +'<script>window.scroll(0,5000000);</script>'
            stdout_file.close()

        return content(proc)
    runBackup._cp_config = {'response.stream': True}

    @cherrypy.expose
    def runCalc(self, **kargs):
        cherrypy.response.headers['Content-Type'] = 'text/html'
        executable = self.getExecutable()
        cmd = executable + ["--calc"]
        #cherrypy.log("Running "+str(cmd), context='', severity=logging.ERROR)
        out_file = codecs.open(os.path.join(self.current_dir, 'stdout_calc.tmp'), 'w', encoding='utf8', buffering=0)

        if bos.isWin32():
            startupinfo = subprocess.STARTUPINFO()
            startupinfo.dwFlags |= subprocess.STARTF_USESHOWWINDOW
            startupinfo.wShowWindow = win32con.SW_HIDE
        
            proc = subprocess.Popen(cmd,
                        bufsize=0,
                        cwd = self.current_dir,
                        stdout=out_file,
                        stderr=subprocess.STDOUT,
                        startupinfo=startupinfo)
        else:
            proc = subprocess.Popen(cmd,
                        bufsize=0,
                        cwd = self.current_dir,
                        stdout=out_file,
                        stderr=subprocess.STDOUT)
 

        def content(proc):
            
            header = """<html>
            <body style="background-color:black;color:white;">
            <p>Calcul en cours</p>

            """             
            yield header
            retval = None
            stdout_file = open(os.path.join(self.current_dir,'stdout_calc.tmp'), 'rb', 0)
            stdout_file.seek(0)
            while retval == None:
                retval = proc.poll()
                where = stdout_file.tell()
                s = stdout_file.read()
                if s:
                    yield s.replace("\n", "<br />\n")+'<script>window.scroll(0,500000);</script>'
                else:
                    time.sleep(1)
                    stdout_file.seek(where)

            s = stdout_file.read()
            yield s.replace("\n", "<br />\n")
            if (retval == 0): mess = 'Ok'
            else: mess = 'Error '+str(retval)
            yield '<hr />' + mess +'<script>window.scroll(0,5000000);</script>'
            stdout_file.close()

        return content(proc)
    runCalc._cp_config = {'response.stream': True}


    @cherrypy.expose
    def runSync(self, **kargs):
        cherrypy.response.headers['Content-Type'] = 'text/html'
        executable = self.getExecutable()
        cmd = executable + ["--sync"]
        #cherrypy.log("Running "+str(cmd), context='', severity=logging.ERROR)
        stdout_file = codecs.open(os.path.join(self.current_dir,'stdout_sync.tmp'), 'w', encoding='utf8', buffering=0)

        if bos.isWin32():
            startupinfo = subprocess.STARTUPINFO()
            startupinfo.dwFlags |= subprocess.STARTF_USESHOWWINDOW
            startupinfo.wShowWindow = win32con.SW_HIDE
        
            proc = subprocess.Popen(cmd,
                        bufsize=0,
                        cwd = self.current_dir,
                        stdout=stdout_file,
                        stderr=subprocess.STDOUT,
                        startupinfo= startupinfo)
        else:
            proc = subprocess.Popen(cmd,
                        bufsize=0,
                        cwd = self.current_dir,
                        stdout=stdout_file,
                        stderr=subprocess.STDOUT)


        def content(proc):
            
            header = """<html>
            <body style="background-color:black;color:white;">
            Synchronisation en cours ...<br />

            """             
            yield header
            retval = None
            stdout_file = open(os.path.join(self.current_dir,'stdout_sync.tmp'), 'rb', 0)
            stdout_file.seek(0)
            while retval == None:
                retval = proc.poll()
                where = stdout_file.tell()
                s = stdout_file.read()
                if s:
                    yield s.replace("\n", "<br />\n")+'<script>window.scroll(0,500000);</script>'
                else:
                    time.sleep(1)
                    stdout_file.seek(where)

            s = stdout_file.read()
            yield s.replace("\n", "<br />\n")
            if (retval == 0): mess = 'Ok'
            else: mess = 'Error '+str(retval)
            yield '<hr />' + mess +'<script>window.scroll(0,5000000);</script>'
            stdout_file.close()

        return content(proc)
    runSync._cp_config = {'response.stream': True}

 
        
    @cherrypy.expose
    def runRestore(self, **kargs):
        # attention, lr passage par l'url se decompose automatiquement en latin1
        params = demjson.decode(kargs['params'], strict=True, encoding='latin1')
        
        if 'same-dest' in params:
            params['destpath']='INPLACE'
        else:
            dpath = params['destpath']
            if dpath[-1] == '\\':
                params['destpath']= dpath[:-1]

        executable = self.getExecutable()
        cmd = []
        for x in executable:
            if " " in x:
                cmd = cmd + ['"'+x+'"']
            else:
                cmd = cmd + [x]
        cmd = cmd + ['--destpath="'+params['destpath']+u'"', '--bid='+params['bid'], '-R']
        
        for to_restore in params['rpath']:
            cmd = cmd + [u'--restore="' + unquote(to_restore)+'"']
        if bos.isWin32():
            scmd = u'"'+u" ".join(cmd)+u'"'
        else:
            scmd = u" ".join(cmd)

        stdout_file = codecs.open(os.path.join(self.current_dir,'stdout_restore.tmp'), 'w', encoding='utf8', buffering=0)

        if bos.isWin32():
            startupinfo = subprocess.STARTUPINFO()
            startupinfo.dwFlags |= subprocess.STARTF_USESHOWWINDOW
            startupinfo.wShowWindow = win32con.SW_HIDE

            proc = subprocess.Popen(scmd.encode(sys.getfilesystemencoding()),
                        stdout=stdout_file,
                        bufsize=0,
                        shell=True,
                        cwd = self.current_dir,
                        stderr=subprocess.STDOUT,
                        startupinfo=startupinfo)
        else:
            proc = subprocess.Popen(scmd.encode(sys.getfilesystemencoding()),
                        stdout=stdout_file,
                        bufsize=0,
                        shell=True,
                        cwd = self.current_dir,
                        stderr=subprocess.STDOUT)


        def content(proc):
            
            header = """<html>
            <body style="background-color:black;color:white;">

            """             
            yield header
            retval = None
            stdout_file = open(os.path.join(self.current_dir,'stdout_restore.tmp'), 'rb', 0)
            stdout_file.seek(0)
            while retval == None:
                retval = proc.poll()
                where = stdout_file.tell()
                s = stdout_file.read()
                if s:
                    yield s.replace("\n", "<br />\n")+'<script>window.scroll(0,500000);</script>'
                else:
                    time.sleep(1)
                    stdout_file.seek(where)
                    
            s = stdout_file.read()
            yield s.replace("\n", "<br />\n")
            if (retval == 0): mess = 'Ok'
            else: mess = 'Error '+str(retval)
            yield '<hr />' + mess +'<script>window.scroll(0,5000000);</script>'
            stdout_file.close()

        return content(proc)
    runRestore._cp_config = {'response.stream': True}
    
    @cherrypy.expose
    def exit(self):
        cherrypy.response.headers['Content-Type'] = 'text/html; charset=UTF-8'
        return self.parseTpl('exit')

    @cherrypy.expose
    def real_exit(self):
        if not bos.isWin32():
            sys.exit(0)
            raise SystemExit(0)

    @cherrypy.expose
    def saveConfig(self, **kargs):
        self.kernel.config.reload() # au cas ou il a chang entre temps
        fls = {}
        fls['include'] = []
        for felem in list(set(demjson.decode(kargs['include'], strict=True, encoding='utf8'))):
            if felem.strip != '':
                fls['include'].append(bos.toOsFilename(felem))

        fls['exclude'] = []
        for felem in list(set(demjson.decode(kargs['exclude'], strict=True, encoding='utf8'))):
            if felem.strip != '':
                fls['exclude'].append(bos.toOsFilename(felem))

        #fls['include'] = filter(lambda x : x.strip() != '', demjson.decode(kargs['include'], strict=True, encoding='utf8'))
        #fls['exclude'] = filter(lambda x : x.strip() != '', demjson.decode(kargs['exclude'], encoding='utf8'))
        fls['excludepattern'] = []
        for xpatt in list(set(kargs['excludepattern'].split("\n"))):
            xpatt = xpatt.replace("\r","")
            if xpatt.strip():
                fls['excludepattern'].append(xpatt.strip())

        f_iter = fileIterator.FileIterator(fls)
        files = self.kernel.config.getSection('files')

        incl = f_iter.getInclude()
        include = []
        include_ajax = []
        for p_inc in incl:
            include.append(bos.toOsFilename(p_inc))
            include_ajax.append({'id':bos.toOsFilename(p_inc), 'path':bos.toOsFilename(p_inc)})
            
        excl = f_iter.getExclude()
        exclude_ajax = []
        exclude = []
        for p_exc in excl:
            exclude.append(bos.toOsFilename(p_exc))
            exclude_ajax.append({'id':bos.toOsFilename(p_exc), 'path':bos.toOsFilename(p_exc)})
        
        files['include'] = include
        files['exclude'] = exclude
        files['excludepattern'] = fls['excludepattern']

        if( kargs['use_vss'] == 'False' and files.has_key('use_vss')):
            del(files['use_vss'])
        elif( kargs['use_vss'] == 'True'):
            files['use_vss'] = 1

        scheduler = self.kernel.config.getSection('scheduler')
        scheduler['hour'] = '%02d' % int(kargs['hour'])
        scheduler['min'] = '%02d' % int(kargs['min'])
        bos.updateLaunchTime(kargs['hour'], kargs['min'])

        security = self.kernel.config.getSection('security')
        if( kargs['max_send_speed'] != '' ):
            security['max_send_speed'] = int(kargs['max_send_speed'])
        elif(security.has_key('max_send_speed') ):
            del(security['max_send_speed'])

        if( kargs['max_recv_speed'] != '' ):
            security['max_recv_speed'] = int(kargs['max_recv_speed'])
        elif(security.has_key('max_recv_speed') ):
            del(security['max_recv_speed'])
        
        cfg = self.kernel.config.getRootSection()['kiwi']
        cfg['shutdown_after_backup'] = not kargs['shutDownAfterBackup'] == 'False'

        data = self.kernel.config.save()
            
        status = 'ok' if data else 'false'
        return demjson.encode({
            'status': status,
            'include': include_ajax,
            'exclude': exclude_ajax,
            'excludepattern': "\n".join(fls['excludepattern'])
        }, encoding = 'utf8')
    
    
    @cherrypy.expose
    def checkports(self):
        cherrypy.response.headers['Content-Type'] = 'application/json; charset=UTF-8'
        results = self.kernel.runCheckports()
        response = []
        
        for k, testedport in results['results'].iteritems():
            port = testedport['portnumber']
            if(testedport['success']):
                response.append( {
                  "status" : "OK",
                  'message' : "Connexion au port "+str(port)+u" réussie",
                  'portnumber' : port
                })
            else:
                response.append( {
                  "status" : "KO",
                  'message' : 'Connexion au port '+str(port)+ u' échouée, erreur : '+testedport['message'],
                  'portnumber' : port
                  })
        return demjson.encode( response, encoding = 'utf8')
      
def cp_update_config(config, CURRENT_DIR):
    listenport = config.getSection('params').get('listenport', 8081)
    listenaddress = config.getSection('params').get('listenaddress', '127.0.0.1')
    print 'starting in '+CURRENT_DIR
    
    
    cherrypy.config.update({
        'environment': 'staging',
        'autoreload.on': False,
        'environment.encoding':'utf8',
        'log.error_file': 'kservice.log',
        'log.tracebacks.on': True,
        'server.socket_host': listenaddress,
        'server.socket_port': listenport,
        'tools.decode.encoding' : 'utf-8',
        'tools.encode.encoding' : 'utf-8',
        'tools.encode.on' : True,
        'log.screen': False
    })
    if bos.isWin32():
        cherrypy.config.update({
            'log.error_file': 'c:\\kservice.log',
        })

    auth_user = config.getSection('params').get('auth_user', None)
    auth_pass = config.getSection('params').get('auth_pass', None)
 
    if auth_user and auth_pass:
        print 'Use HTTP Authentification'
        cherrypy.config.update({
            'tools.basic_auth.on': True,
            'tools.basic_auth.realm': "Kiwi GUI",
            'tools.basic_auth.users': {auth_user:md5.new(auth_pass).hexdigest()}
        })

    CONF = {
        '/': {
            'tools.staticdir.root': os.path.join(CURRENT_DIR, 'webclient', 'static')

        },
        '/css': {
            'tools.staticdir.on': True,
            'tools.staticdir.dir': 'css'
        },
        '/images': {
            'tools.staticdir.on': True,
            'tools.staticdir.dir': 'images'
        },
        '/js': {
            'tools.staticdir.on': True,
            'tools.staticdir.dir': 'js'
        }
    }
    return CONF

def unquote(text):
    def unicode_unquoter(match):
        return unichr(int(match.group(1),16))
    return re.sub(r'%u([0-9a-fA-F]{4})',unicode_unquoter,text)


if __name__ == '__main__':
    try:
        os.setuid(0)
    except:
        print "not root"
    
    if bos.isOsX() and os.getuid():
        from Authorization import Authorization, kAuthorizationFlagDestroyRights
        try:
            auth = Authorization(destroyflags=(kAuthorizationFlagDestroyRights,))
            pipe = auth.executeWithPrivileges(os.getcwd()+"/../MacOS/server")
            sys.stdout.write(pipe.read())
            pipe.close()
        finally:
            print "close"
    else:

        CURRENT_DIR = bos.getCurrentDir()

        CPATH = ''
        if len(sys.argv) > 1 and bos.isOsX():
            import kclient.kiwiCmd 
            kclient.kiwiCmd.runKiwi()
            sys.exit(0)

        else:
            CPATH = CURRENT_DIR+'/kiwi.conf'
            print 'Using default config file'
 
        config = configManager.ConfigManager()
        config.load(CPATH)

        conf = cp_update_config(config, CURRENT_DIR)

        kapp = KiwiManager(CPATH, CURRENT_DIR)
    
        cherrypy.config.update(conf)
        cherrypy.tree.mount(kapp, '/', conf)
        cherrypy.engine.start()
    
        if bos.isOsX():
            import webbrowser
            webbrowser.open('http://localhost:8082')

        cherrypy.engine.block()
